// Copyright (c) 2002 Andreas Jnsson, www.AngelCode.com

// Don't forget to link with winhook.lib

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <math.h>
#include "resource.h"
#include "winhook/winhook.h"

// Global Variables:
HWND    hDesktopWnd = 0;
HWND    hWnd        = 0;
HRGN    updateRgn   = 0;
int     posX        = 0;
int     posY        = 0;
float   angle       = 0;

// Foward declarations of functions included in this code module:
BOOL			 WindowCreate(HINSTANCE);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK DialogProc(HWND, UINT, WPARAM, LPARAM);

void             DrawDesktop();

// Main function
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	// Create our window:
	if (!WindowCreate(hInstance)) 
		return FALSE;

	// Create the GDI objects
	updateRgn = CreateRectRgn(0, 0, 0, 0);

	// Find the desktop window
	HWND hProgMan  = FindWindow("ProgMan", NULL);	
	HWND hShellDef = FindWindowEx(hProgMan, NULL, "SHELLDLL_DefView", NULL);
	hDesktopWnd    = FindWindowEx(hShellDef, NULL, "SysListView32", NULL);

	// Install our hook procedure
	int r = SetHook(hDesktopWnd, hWnd);
	if( r < 0 )
		MessageBox(hWnd, "Failed to install hook", "Error", MB_OK);

	// Main message loop:
	MSG msg;
	while( GetMessage(&msg, NULL, 0, 0) ) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// Do a last invalidation to remove the drawing
	RECT rc = {posX, posY, posX+100, posY+100};
	InvalidateRect(hDesktopWnd, &rc, TRUE);

	// Clean up
	RemoveHook();
	DeleteObject(updateRgn);

	return msg.wParam;
}

// Window creation code
BOOL WindowCreate(HINSTANCE hInstance)
{
	// Create our dialog that we will use for main window
	hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG), 0, DialogProc);
	if (!hWnd)
	{
		char *buff;
		DWORD err = GetLastError();
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, 0, err, 0, (char*)&buff, 256, 0);

		MessageBox( NULL, (LPCTSTR)buff, "Error", MB_OK | MB_ICONINFORMATION );

		LocalFree( buff );

		return FALSE;
	}

	// We don't want to show our window
	ShowWindow(hWnd, SW_SHOW);

	return TRUE;
}

// Our dialog procedure
INT_PTR CALLBACK DialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
	case WM_COMMAND:
		// Close the window no matter what command it is
		DestroyWindow(hWnd);
		return TRUE;

	case WM_INITDIALOG:
		{
			// Set the small icon
			HICON iconSmall = (HICON)LoadImage(GetModuleHandle(0), MAKEINTRESOURCE(IDI_SMALL), IMAGE_ICON, 0, 0, LR_DEFAULTSIZE|LR_SHARED);
			SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)iconSmall);

			// Create a timer so that we can animate our drawing
			SetTimer(hWnd, 1, 50, 0);
		}
		return TRUE;

	case WM_CLOSE:
		// Destroy the window
		DestroyWindow(hWnd);
		return TRUE;

	case WM_DESTROY:
		// Kill our timer
		KillTimer(hWnd, 1);

		// Post the quit message so that our main loop will know that we have quit
		PostQuitMessage(0);
		return TRUE;

	case WM_APP+100:
		// This is where we handle the messages sent by our hook procedures
		if( wParam == WM_PAINT && lParam == 1 )
		{
			// Get the update region before the desktop 
			// window handles the WM_PAINT message
			GetUpdateRgn(hDesktopWnd, updateRgn, FALSE);
		}
		else if( wParam == WM_ERASEBKGND && lParam == 0 )
		{
			// Draw our image before the desktop window draws the icons
			DrawDesktop();
		}
		return TRUE;

	case WM_TIMER:
		// Animate the drawing
		{
			angle += 0.1f;
			if( angle >= 2*3.141592f ) angle -= 2*3.141592f;

			// Get the current mouse position
			POINT pt;
			GetCursorPos(&pt);

			// Update position
			RECT rc = {posX-50, posY-50, posX+50, posY+50};
			InvalidateRect(hDesktopWnd, &rc, TRUE);

			posX = pt.x;
			posY = pt.y;

			RECT rc2 = {posX-50, posY-50, posX+50, posY+50};
			InvalidateRect(hDesktopWnd, &rc2, TRUE);
		}
		return TRUE;
   }

   return FALSE;
}

// This is where we draw our image on the desktop
void DrawDesktop()
{
	// Get the DC to the desktop window
	HDC dc = GetDC(hDesktopWnd);

	// Store DC settings
	int storedDC = SaveDC(dc);

	// Set the clipping region
	SelectObject(dc, updateRgn);

	// Draw the rotating star
	POINT pts[5];
	pts[0].x = posX + int(50*cos(angle + 0.000)); pts[0].y = posY + int(50*sin(angle + 0.000));
	pts[1].x = posX + int(50*cos(angle + 2.513)); pts[1].y = posY + int(50*sin(angle + 2.513));
	pts[2].x = posX + int(50*cos(angle + 5.027)); pts[2].y = posY + int(50*sin(angle + 5.027));
	pts[3].x = posX + int(50*cos(angle + 1.257)); pts[3].y = posY + int(50*sin(angle + 1.257));
	pts[4].x = posX + int(50*cos(angle + 3.770)); pts[4].y = posY + int(50*sin(angle + 3.770));
	Polygon(dc, pts, 5);

	// Restore DC settings to their original values
	RestoreDC(dc, storedDC);

	// Release the DC
	ReleaseDC(hDesktopWnd, dc);
}